
# This is a test of whether the VM can handle recursive
# arrays, that is, arrays which contain references to themselves.
# If arrays are by-value types, then adding an array into itself
# will just copy the array. But if arrays are by-reference,
# this could create a recursive situation.
#
# The operations which really need to be checked for recursion are:
#  acmp    array comparison
#  afind   finding an array within another array
#  asort   sorting array elements in ascending order

DOUBLE

dbl:	1.9

STRING

yes:	"passed\n"
no:	"failed\n"

CODE

	icopy	12, r0
	anew	[r0]		# r0->[]
	ainsert	0, 0, [r0]	# r0->[0]
	ainsert 1, [r0], [r0]	# r0->[0,r0]

	icopy	13, r1
	anew	[r1]		# r1->[]
	ainsert	0, 1, [r1]	# r1->[1]
	ainsert	2, [r1], [r0]	# r0->[0,r0,[1]]
	ainsert	1, [r0], [r1]	# r1->[1,r0]
	ainsert	3, 7, [r0]	# r0->[0,r0,[1,r0],7]

test1:

	icopy	12, r2
	anew	[r2]		# r2->[]
	ainsert	0, 2, [r2]	# r2->[2]
	ainsert	1, [r2], [r2]	# r2->[2,r2]

	icopy	13, r3
	anew	[r3]		# r3->[]
	ainsert	0, 2, [r3]	# r2->[2]
	ainsert 1, [r3], [r3]	# r3->[2,r3]

	acmp	[r2], [r3]	# possibility of recursion here

	bne	failed1
	icopy	yes, r10
	sprint	[r10]
	bra	test2

failed1:
	icopy	no, r10
	sprint	[r10]

test2:

	afind	[r2], [r3], r4	# possibility of recursion here
	icmp	r4, 1

	bne	failed2
	icopy	yes, r10
	sprint	[r10]
	bra	test3

failed2:
	icopy	no, r10
	sprint	[r10]

test3:

	icopy	dbl, r5
	ainsert	2, [r5], [r2]	# r2->[2,r2,1.5]
	ainsert	3, [r3], [r2]	# r2->[2,r2,1.5,r3]

	asort	[r2], [r3]	# possibility of recursion here

	icopy	77, r4

	aindex	0, [r3], [r4]	# r4->1.5
	dcmp	[r4], [r5]
	bne	failed3

	aindex	1, [r3], [r4]	# r4->2
	icmp	[r4], 2
	bne	failed3

	aindex	2, [r3], [r4]	# r4-> r2 or r3

	icopy	yes, r10
	sprint	[r10]

	bra	end

failed3:
	icopy	no, r10
	sprint	[r10]

end:

	halt

